      PROGRAM HRW
C
C     Herwig main program for HzTool/JetWeb.
C     N.B. Use only for unweighted running!
C

c      IMPLICIT NONE
c     HERWIG common blocks etc., includes "IMPLICIT NONE" 
c     so need to come first. ???
#include "hzhrwsteer.inc"

c     HBOOK stuff
      INTEGER NWPAWC,H
      PARAMETER (NWPAWC=9000000)
      COMMON/PAWC/H(NWPAWC)

      EXTERNAL HWUDAT

c     Counter used in event loop
      INTEGER EVLOOP, NUMEV
      INTEGER ILOOP

c     Total cross section
      REAL XSECLOC

c     Mean charged multiplicity
      DOUBLE PRECISION nchavg

c     Temporary variables for output of HZNEVT
      INTEGER HZNEV2
      REAL HZLUMI2

c     HERWIG process type
      INTEGER IPROCLOC, JPROC

c     Event type: 1 = pbar-p, 2 = ep, 3 = ee
c      INTEGER ITYPLOC

c     Offset for IFLAG
      INTEGER IFLOFF

c     HERWIG particle ID number and name
      INTEGER iwig
      CHARACTER*8 nwig

c     Author groups for PDF sets. Should move somewhere more sensible. ???
      CHARACTER*8 NPDFAUT(7)
      CHARACTER*8 PPDFAUT(9)

c     PDF IDs for beam particles, and function to return these
      INTEGER HZPDFSEL

      LOGICAL abort

c     JM: buffer for hwigin output

      character*80 hwibuf(11)
      character*5 versn

      common /hwvrsn/ versn

      integer lenocc

c     Author groups for PDF sets. Should move somewhere more sensible. ???
c     Proton authors
      DATA NPDFAUT /'DO','DFLM','MRS','CTEQ','GRV','ABFOW','BM'/
c     Photon authors
      DATA PPDFAUT /'NN','NN','LAC','NN','GRVph','NN','NN','WHIT-G'
     $     ,'SaSph'/

c     HEPEVT prime common block - included in HERWIG include file
c#include "hepevtp.inc"

c     Misleadingly named common block
c     Contains information on generator, no. of events etc.
#include "heracmn.inc"

c     Common block with steering parameters
#include "hzcontrol.inc"
#include "hzhrwcontrol.inc"

c     Function to calculate mean multiplicity for HERWIG
c     Move to common block, e.g. HZFUNC?
      DOUBLE PRECISION HZHWAVRG

      PRINT*,''
      PRINT*,'*******************************************************'
      PRINT*,'*'
      PRINT*,'* jetweb_hrw.fpp'/
     $     /'$Date: 2007-01-14 22:26:56 +0000 (Sun, 14 Jan 2007) $'  
      PRINT*,'*'
      PRINT*,'*******************************************************'
      PRINT*,''

      GEN = 'HRW'

c     Read steering cards etc.

      write(*,*)'hzlumi = ', hzlumi, ' hzlumi2 = ',hzlumi2

      CALL HZCARDS(1)

      write(*,*)'hzlumi = ', hzlumi, ' hzlumi2 = ',hzlumi2

      CALL HZHRWCARDS
      
      write(*,*)'hzlumi = ', hzlumi, ' hzlumi2 = ',hzlumi2

      CALL HZCARDS(2)

      write(*,*)'hzlumi = ', hzlumi, ' hzlumi2 = ',hzlumi2

c     Set beam particles (and convert to upper case using KERNLIB util)
c     Swap order from steering since event is rotated by pi radians
c     after generation by routine HZHRWCOP. First beam in steering
c     should be one with +ve pz after rotation. But the first beam for
c     HERWIG has to be the photon, if we are doing photoproduction!
      PART1 = hzb2
      PART2 = hzb1
      CALL CLTOU(PART1)
      CALL CLTOU(PART2)

c     Set beam momenta
      PBEAM1 = hzpb2
      PBEAM2 = hzpb1

c     Need to know event type and whether direct or singly/double resolved so can add
c     offset to iflag.

      if ((PART1.eq.'PBAR'.and.PART2.eq.'P').or.
     &     (PART2.eq.'PBAR'.and.PART1.eq.'P')) then
c         ityploc = 1
         write (*,*) '*** jetweb_hrw: pbar-p running'
      else if (((PART1.eq.'E+'.or.PART1.eq.'E-').and.PART2.eq.'P').or.
     &        ((PART2.eq.'E+'.or.PART2.eq.'E-').and.PART1.eq.'P')) then
c         ityploc = 2
         write (*,*) '*** jetweb_hrw: ep running'
      else if ((PART1.eq.'E+'.and.PART2.eq.'E-').or.
     &        (PART2.eq.'E+'.and.PART1.eq.'E-')) then
c         ityploc = 3
         write (*,*) '*** jetweb_hrw: e+e- running'
      else
c         ityploc = 0
         WRITE(*,*) '*** Warning in jetweb_hrw: unknown collision type'
      end if

      DO jproc=1,hzmaxproc

c     Set first process
         IPROC = hziproc(jproc)

         IF (IPROC.NE.-999) THEN

            write(hzunit,*) 'iproc=',iproc
            IPROCLOC = mod(iproc,10000)

            IF (hziproc(2).ne.-999) THEN
c     Set the offset on the histogram flag.
               ifloff = jproc*1000
            ELSE
c     Special case - only one process, there should be no offset on the histogram flag.
               ifloff = 0
            ENDIF

c     if (ityploc.eq.2) then
c     ep direct
c         if (iprocloc.eq.5000.or.iprocloc.eq.9130) ifloff = 1000
c     ep resolved
c         if (iprocloc.eq.1500) ifloff = 2000
c      else if (ityploc.eq.3) then
c     ee direct
c         if (iprocloc.eq.6000) ifloff = 1000
c     ee singly resolved
c         if (iprocloc.eq.5000) ifloff = 2000
c     ee doubly resolved
c         if (iprocloc.eq.1500) ifloff = 3000
c      end if
            write (*,*) '*** jetweb_hrw: iflag offset = ',ifloff

C     Initialize HERWIG
C     Has to be done before setting of parameters

c     JM: redirect unit 6 to a buffer so we can extract the version of herwig
c         from the message that hwigin prints and write it in the xml
            
            open(6, file="hwigin.tmp")

            CALL HWIGIN

c     JM: now redirect unit 6 back to stdout
c         assumes stdout is where it should be!
            
            close(6)
            open(6, file="/dev/stdout")

            open(66, file="hwigin.tmp", status='old')

c     fortran I/O is a nightmare!

            read(66, '(a)')hwibuf

c            write(*,*)'writing hwibuf'
            write(*,*)hwibuf

c            write(*,*)lenocc(hwibuf)

            rewind(66)

            read(66, *)hwibuf
          
            close(66)

            versn = hwibuf(2)

c            close(6)

            CALL JIMMIN

c     Set PTJIM
            PTJIM=HZPTJIM

c     Choose JIMMY running mode
            JMUEO=HZJMUEO

c     Set minimum PT for hard subprocess (only if set in card file)
            if(hzptmin.gt.-998.) ptmin = hzptmin

c     Set PDFs of beam particles
c Use LHAPDF
c            PDFI = HZPDFSEL(2)
c            IF (PART1.eq.'P'.or.PART1.eq.'PBAR') THEN
c               AUTPDF(1) = NPDFAUT(PDFI/1000)
c            ELSE 
c               AUTPDF(1) = PPDFAUT(PDFI/1000)
c            ENDIF
c            MODPDF(1) = MOD(PDFI,1000)
            AUTPDF(1) = 'HWLHAPDF'
            MODPDF(1) = HZPDFSEL(2)

            write(*,*) modpdf(1), autpdf(1), part1 

c            PDFI = HZPDFSEL(1)
c            IF (PART2.eq.'P'.or.PART2.eq.'PBAR') THEN
c               AUTPDF(2) = NPDFAUT(PDFI/1000)
c            ELSE 
c               AUTPDF(2) = PPDFAUT(PDFI/1000)
c            ENDIF
c            MODPDF(2) = MOD(PDFI,1000)

            AUTPDF(2) = 'HWLHAPDF'
            MODPDF(2) = HZPDFSEL(1)
            write(*,*) modpdf(2), autpdf(2), part2 

c     Reduced printout from LHAPDF.
            CALL SetLHAPARM('LOWKEY')

ccc  Set HERWIG-specific parameters from steering

c  Proton and photon radius
            IF (HZPRRAD.GT.-999.) THEN
               JMRAD(73)=DBLE(HZPRRAD) 
               JMRAD(75)=DBLE(HZPRRAD) 
               JMRAD(91)=DBLE(HZPRRAD) 
               JMRAD(93)=DBLE(HZPRRAD) 
            ENDIF
            IF (HZPHRAD.GT.-999.) THEN
               JMRAD(59) =DBLE(HZPHRAD) 
               JMRAD(121)=DBLE(HZPHRAD) 
               JMRAD(127)=DBLE(HZPHRAD) 
            ENDIF
            
            IF (HZQSPAC.GT.-999.)  QSPAC=DBLE(HZQSPAC) 
            IF (HZCLMAX.GT.-999) CLMAX = HZCLMAX
            IF (HZCLPOW.GT.-999) CLPOW = HZCLPOW
            IF (HZPSPLT1.GT.-999) PSPLT(1) = HZPSPLT1
            IF (HZPSPLT2.GT.-999) PSPLT(2) = HZPSPLT2
            IF (HZPHAD.GT.-999) PHAD = HZPHAD
            IF (HZPRSOF.GT.-999) PRSOF = HZPRSOF
            IF (HZBTCLM.GT.-999) BTCLM = HZBTCLM
            IF (HZQCDLAM.GT.-999.) QCDLAM=DBLE(HZQCDLAM) 
            IF (HZVQCUT.GT.-999) VQCUT = DBLE(HZVQCUT)
            IF (HZVGCUT.GT.-999) VGCUT = DBLE(HZVGCUT)
            IF (HZVPCUT.GT.-999) VPCUT = DBLE(HZVPCUT)
            IF (HZPTRMS.GT.-999.)  PTRMS=DBLE(HZPTRMS) 
            
            IF (HZIOPREM.GT.-999) IOPREM = HZIOPREM
            IF (HZOMEGA0.GT.-999) OMEGA0 = DBLE(HZOMEGA0)
            IF (HZASFIXD.GT.-999) ASFIXD = DBLE(HZASFIXD)
            IF (HZQ2WWMX.GT.-999) Q2WWMX = DBLE(HZQ2WWMX)

            IF (HZPDFPOW.GT.-999) PDFPOW = DBLE(HZPDFPOW)
            IF (HZPDFX0.GT.-999) PDFX0 = DBLE(HZPDFX0)
            
            IF (HZISPAC.GT.-999)   ISPAC=HZISPAC 
            IF (HZIFLMAX.GT.-999)  IFLMAX=HZIFLMAX
            IF (HZNFLAV.GT.-999)   NFLAV=HZNFLAV

ccc  Parton distributions ???

ccc  JIMMY multiple interaction flag 
            MSFLAG = HZJIMMY

c     Set initial seed for random number generator
            NRN(1)=hzrnd1
            NRN(2)=hzrnd2
            WRITE(*,*) 'Using random number sequence ',NRN(1),', ',NRN(2
     $           )
            
c     Compute parameter-dependent constants
            CALL HWUINC
            
c     Switch off decays of final-state particles selected to be stable
            DO 23 iloop = 1, 200
               IF (HZSTAB(iloop).ne.-9999) THEN
                  WRITE(*,*) 'Making particle stable: ',HZSTAB(iloop)
c     Convert PDG code to HERWIG particle name and set to be stable
                  CALL HWUIDT(1,HZSTAB(iloop),IWIG,NWIG)
                  CALL HWUSTA(NWIG)
               ENDIF
 23         CONTINUE
            
c     Initialize elementary processes
            CALL HWEINI
            IF (MSFLAG.EQ.1) CALL JMINIT
      
c     Get total cross section from HERWIG in nanobarns. Convert to pb.
            XSECLOC = AVWGT*1000

c     Decide how many events to generate and corresponding luminosity

            write(*,*)'hzlumi = ', hzlumi, ' hzlumi2 = ',hzlumi2

            CALL HZNEVT(XSECLOC, HZNEV, HZLUMI, HZNEV2, HZLUMI2)

            write(*,*)'hzlumi = ', hzlumi, ' hzlumi2 = ',hzlumi2
            
c     MAXEV is used by Herwig for number of events, MAXER = max. no. of errors before dying
            MAXEV = HZNEV2
            MAXER = MAX(MAXEV/5,100)
c     Save value of MAXEV in HZNEW
            HZNEV = MAXEV            

            IF (jproc.EQ.1) THEN
c     Calculate mean charged multiplicity, needed by some analysis routines
c     NB. The first process is assumed to give a decent estimate of the average.
              write (6,*)
     &           'Generating a few events to estimate mean multiplicity'
               nchavg = HZHWAVRG(5000,msflag)
            ENDIF
            
c     Initialize HBOOK histogramming
            CALL HZHISTO(1+ifloff,nchavg)
                                    
c     Generate events
            write (6,*) 'Starting main event generation loop'
            NUMEV = 0

            write(*,*) 'Number of events to generate is ', MAXEV

            DO EVLOOP=1,MAXEV
 50            CONTINUE
c     Initialize event
               CALL HWUINE
c     Generate hard subprocess
               CALL HWEPRO
c     Generate parton cascades
               CALL HWBGEN
ccc   premserr ???
               
c     Generate multiparton interactions
               IF (MSFLAG.EQ.1) THEN
                  CALL HWMSCT(abort)
                  IF (abort) GOTO 50
               ENDIF
               
C---  DO HEAVY OBJECT DECAYS ???
c#ifdef HERWIG59
c               CALL HWDHQK
c#else
               CALL HWDHOB
c#endif
C---  DO CLUSTER HADRONIZATION
               CALL HWCFOR
C---  DO CLUSTER DECAY
               CALL HWCDEC
C---  DO UNSTABLE PARTICLE DECAYS
               CALL HWDHAD
C---  DO HEAVY FLAVOUR DECAYS
               CALL HWDHVY
C---  ADD SOFT UNDERLYING EVENT IF NEEDED
               CALL HWMEVT
C---  FINALISE EVENT
               CALL HWUFNE
               
               IF (IERROR.NE.0) GOTO 50
               
c     Transfer information to HEPEVT common, HERACMN etc.
               CALL HZFILHEP
               
c     Set event weight WTX as some analysis routines seem to use this
c     Only dealing with unweighted events at the moment
               WTX = 1.0
               
               NUMEV = NUMEV + 1
               
c     Fill histograms
               CALL HZHISTO(2+ifloff,nchavg)
               
            ENDDO
            
C     Finish off Herwig Routines
            CALL HWEFIN
            IF (MSFLAG.EQ.1) CALL JMEFIN
            
c     Set number of events and cross section (in nb) in HERACMN
            NTOT = NUMEV
            XSEC = AVWGT
            
c     Finish off HBOOK stuff
            CALL HZHISTO(3+ifloff,nchavg)

c     Start XML file (only for the first process)
            IF (hzxmlout.ne.0.AND.jproc.EQ.1) THEN
               CALL HZHRWXML(0)
               CALL HZHRWXML(1)
               CALL HZHRWXML(2)
               CALL HZHRWXML(3)
            ENDIF

c     Write XML run information for each process
            IF (hzxmlout.ne.0) THEN
               CALL HZHRWXML(4)
            ENDIF
           
c     IPROC not set
         ENDIF

c     Next process
      ENDDO

c     Write histograms in aida format
      IF (hzaidaout.NE.0) THEN
         CALL HZAIDA
      ENDIF

c     Output histograms and close file.
      CALL HZHISTO(-1,nchavg)

      IF (hzxmlout.NE.0) THEN
c     Finish and close XML file
         CALL HZHRWXML(-3)
         CALL HZHRWXML(-1)
      ENDIF

      PRINT*,''
      PRINT*,'*******************************************************'
      PRINT*,'*'
      PRINT*,'* Outputted and ended'    
      PRINT*,'*'
      PRINT*,'*******************************************************'
      PRINT*,''

      END


c     Dummy subroutine to keep HERWIG happy
      SUBROUTINE hwaend
      RETURN
      END
 
