      SUBROUTINE HZPYTXML(IFLAG)
c
c     Write out PYTHIA parameters in XML format
c
c     IFLAG = 0 :  Write XML header
c             1 :  Open root element
c            -1 :  Close root element
c             2 :  Write reaction element
c             3 :  Open runSeries element
c            -3 :  Close runSeries element
c             4 :  Write run element (may be called several times)
c
c     Write out PYTHIA parameters in XML format
c
c     Call HZXML before and after this to do generator-independent bits
c
      IMPLICIT  NONE
c
c  Pythia's include files common block declarations and
c  array dimensions, but do not declare variable types.
c  Hence declare types here since using "implicit none" in HzTool.
c

C     pyjets.inc
      INTEGER N,NPAD,K
      DOUBLE PRECISION P, V
      COMMON/PYJETS/N,NPAD,K(4000,5),P(4000,5),V(4000,5)
      SAVE /PYJETS/

C     pypars.inc
      INTEGER MSTP,MSTI
      DOUBLE PRECISION PARP,PARI
      COMMON/PYPARS/MSTP(200),PARP(200),MSTI(200),PARI(200)
      SAVE /PYPARS/

C     pysubs.inc
      INTEGER MSEL, MSELPD, MSUB, KFIN
      DOUBLE PRECISION CKIN
      COMMON/PYSUBS/MSEL,MSELPD,MSUB(500),KFIN(2,-40:40),CKIN(200)
      SAVE /PYSUBS/

C     pydatr.inc
      INTEGER MRPY
      DOUBLE PRECISION RRPY
      COMMON/PYDATR/MRPY(6),RRPY(100)
      SAVE /PYDATR/

C     pydat1.inc
      INTEGER MSTU, MSTJ
      DOUBLE PRECISION PARU, PARJ
      COMMON/PYDAT1/MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      SAVE /PYDAT1/

C     pydat3.inc
      INTEGER MDCY, MDME, KFDP
      DOUBLE PRECISION BRAT
      COMMON/PYDAT3/MDCY(500,3),MDME(8000,2),BRAT(8000),KFDP(8000,5)
      SAVE /PYDAT3/

C     pyint5.inc
      INTEGER NGENPD, NGEN
      DOUBLE PRECISION XSEC
      COMMON/PYINT5/NGENPD,NGEN(0:500,3),XSEC(0:500,3)
      DOUBLE PRECISION SIGT
      COMMON/PYINT7/SIGT(0:6,0:6,0:5)

C     PYINT1
      INTEGER MINT
      DOUBLE PRECISION VINT
      COMMON/PYINT1/MINT(400),VINT(400)

C     Doesn't seem to be declared anywhere else
      INTEGER PYCOMP


c#include "hzcontrol.inc"
c#include "heracmn.inc"

      INTEGER IFLAG,IUNIT
      PARAMETER (IUNIT=12)

c     File stem for output files
      CHARACTER*80 stem

c     Buffer for formatting numbers in output
      CHARACTER*14 buff
      INTEGER buflen
      PARAMETER (buflen=14)
      
c     Function to find number of occupied characters in a string (KERNLIB)
      INTEGER lenocc

c     Lower-case versions of beam particle names
      CHARACTER*8 part1s, part2s

c     Name of PDF author group
      CHARACTER*20 PDFAUTS

      CHARACTER*16 CHAU

c     Convert beam particle names to lower case: OK for p, pbar, e+, e-
      CALL PYNAME(MINT(11),CHAU)
      part1s=CHAU(1:12)
      CALL PYNAME(MINT(12),CHAU)
      part2s=CHAU(1:12)
      CALL CUTOL(part1s)
      CALL CUTOL(part2s)

      if(MINT(141).ne.0) then !this means it is a photon inside lepton beam
         part1s = 'gamma/e+'
      endif

      if(MINT(142).ne.0)then
         part2s = 'gamma/e+'
      endif

c      write(*,*) 'JM: particle types are: ',part1s, ' ',part2s


      if (IFLAG.eq.0) then
c     File stem is first command-line argument
         call GETARG(1,stem)
c     Open file for XML output
         open(unit=iunit,file=stem(1:LENOCC(stem))//'.xml',status='new')
c     JMB need to do this with a fortran statement, otherwise you get a
c     leading space which makes JDOM barf (!)
         write(iunit,1000) 
 1000    FORMAT('<?xml version="1.0" encoding="ISO-8859-1" ?>')

      elseif (IFLAG.eq.1) then
         write (iunit,*) '<hep:hepml'//
     &     ' xsi:schemaLocation='//
     &     '"http://www.cedar.ac.uk/hepml/0.1/ pythia.xsd"'//
     &     ' xmlns:hep="http://www.cedar.ac.uk/hepml/0.1/"'//
     &     ' xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">'
      elseif (IFLAG.eq.-1) then
         write (iunit,*) '</hep:hepml>'
         close(unit=iunit)

      elseif (IFLAG.eq.2) then
         write (iunit,*) ' <hep:reaction>'
         write (iunit,*) '  <hep:collision>'

c     ??? Have commented out HERWIG stuff but not writted PYTHIA stuff!

         write (iunit,*) '   <hep:particle name="'//
     &        PART1S(1:LENOCC(PART1S))//'" id="1">'
         write (buff,'(F14.4)') p(1,3)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:momentum unit="GeV">'//
     &        buff(1:lenocc(buff))//'</hep:momentum>'
         write (iunit,*) '   </hep:particle>'

         write (iunit,*) '   <hep:particle name="'//
     &        PART2S(1:LENOCC(PART2S))//'" id="2">'
         write (buff,'(F14.4)') -p(2,3)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:momentum unit="GeV">'//
     &        buff(1:lenocc(buff))//'</hep:momentum>'
         write (iunit,*) '   </hep:particle>'

         write (iunit,*) '  </hep:collision>'
c     write (iunit,*) '   <hep:comment>No Comment!</hep:comment>'
         write (iunit,*) ' </hep:reaction>'

      elseif (IFLAG.eq.3) then
         write (iunit,*) ' <hep:runSeries>'
      elseif (IFLAG.eq.-3) then
         write (iunit,*) ' </hep:runSeries>'

      elseif (IFLAG.eq.4) then
         write (iunit,*) '  <hep:run>'

         write (iunit,*) '   <hep:randomSeed>'
         write (buff,'(I14)') mrpy(1)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:seed index="1">'//
     &        buff(1:lenocc(buff))//'</hep:seed>'
c         write (buff,'(I14)') hzrnd2
c         call cleft(buff,1,buflen)
c         write (iunit,*) '    <hep:seed index="2">'//
c     &        buff(1:lenocc(buff))//'</hep:seed>'
         write (iunit,*) '   </hep:randomSeed>'

         write (iunit,*) '   <hep:normalisation>'

         write (buff,'(E14.7)') 1E+09*XSEC(0,3)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:crossSection unit="pb">'//
     &        buff(1:lenocc(buff))//'</hep:crossSection>'

         write (buff,'(I14)') MSTI(5)
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:numberOfEvents>',
     &        buff(1:lenocc(buff))//'</hep:numberOfEvents>'

         write (buff,'(E14.7)') MSTI(5)/(1E+09*XSEC(0,3))
         call cleft(buff,1,buflen)
         write (iunit,*) '    <hep:luminosity unit="pb^-1">'//
     &        buff(1:lenocc(buff))//'</hep:luminosity>'

         write (iunit,*) '   </hep:normalisation>'

         write (iunit,*) '   <hep:generatorSettings>'
c     ??? PYTHIA version hardcoded -- temporary bodge
         write (iunit,99) mstp(181),mstp(182)
 99      FORMAT('    <hep:generator name="PYTHIA" version="',I1,'.'
     &   ,I3,'"/>')

         PDFAUTS='PYLHAPDF'
         write(buff, *)mstp(51)
         write (iunit,98) 
     &        PDFAUTS(1:LENOCC(PDFAUTS)),buff(2:LENOCC(buff))

         write (iunit,*) '     <hep:particle name="'//
     &        PART1S(1:LENOCC(PART1S))//'"/>'
         write (iunit,*) '    </hep:pdf>'
c     Only write second PDF if beam particles are different
         if (PART1S.NE.PART2S) then
            PDFAUTS = 'PYLHAPDF'

            write(buff, *) mstp(55)
            write (iunit,98) 
     &           PDFAUTS(1:LENOCC(PDFAUTS)),buff(2:LENOCC(buff))
            write (iunit,*) '     <hep:particle name="'//
     &           PART2S(1:LENOCC(PART2S))//'"/>'
            write (iunit,*) '    </hep:pdf>'
         endif
 98      FORMAT('    <hep:pdf set="',A, '" member="',
     &        A, '" scheme="LHAPDF">')

c     Double arrays first
         call hzxmlda(iunit,'CKIN',200,CKIN)
         call hzxmlda(iunit,'PARP',200,PARP)
         call hzxmlda(iunit,'PARJ',200,PARJ)
c     Integer arrays second
         call hzxmlia(iunit,'MSTP',200,MSTP)
         call hzxmlia(iunit,'MSTJ',200,MSTJ)
         call hzxmlia(iunit,'MSUB',500,MSUB)

c     Double matrices first

c     Integer matrices first
         call hzxmlim(iunit,'KFIN',1,2,2,-40,40,81,KFIN)

         write (iunit,*) '   </hep:generatorSettings>'

         write (iunit,*) '  </hep:run>'

      endif

      RETURN
      END
